package hterhors.editor;

import hterhors.editor.markers.IsrRuleValidator;

import java.util.ArrayList;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;

/**
 * @author  Hendrik
 */
class ISRContentAssistantDefaultProcessor implements IContentAssistProcessor {
	private final IContextInformation[] NO_CONTEXTS = {};
	private final char[] PROPOSAL_ACTIVATION_CHARS = { 's', 'f', 'p', 'n', 'm', };
	private ICompletionProposal[] NO_COMPLETIONS = {};
	private int override = 0;
	private String preSpace = "";
	protected static final String lineSeparator = System
			.getProperty("line.separator");

	/**
	 * Use this method to set the size for overriding by using content-assistant.
	 * @param override  The size to be override.
	 * @uml.property  name="override"
	 */
	public void setOverride(int override) {
		this.override = override;
	}

	@Override
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
			int offset) {
		try {
			IDocument document = viewer.getDocument();
			ArrayList<CompletionProposal> result = new ArrayList<CompletionProposal>();
			if (!IsrRuleValidator.startSymbolExists(document.get())) {
				result.add(new CompletionProposal(
						preSpace + "$$S:  ;" + lineSeparator,
						offset,
						override,
						5 + preSpace.length(),
						null,
						"New start-symbol",
						null,
						"The '$$S' symbol is the start symbol and must be included\n"
								+ "in the grammar. Additionally, the '$$SS' symbol is predefined and\n"
								+ "must not be used in the grammatic description. "));
			}
			for (String nonterminal : IsrRuleValidator
					.getMissingNontermDec(document.get())) {
				result.add(new CompletionProposal(preSpace + "$" + nonterminal
						+ ":  ;" + lineSeparator, offset, override, 3
						+ nonterminal.length() + preSpace.length(), null,
						nonterminal + " - Missing!", null,
						"Add the missing nonterminal-declaration ($"
								+ nonterminal + ":) to the grammar."));
			}
			result.add(new CompletionProposal(
					preSpace + "$:  ;" + lineSeparator,
					offset,
					override,
					1 + preSpace.length(),
					null,
					"New declaration",
					null,
					"\"NONTERMINAL\" symbols are defined equally as TERMINAL symbols but\n"
							+ "must have a '$' or '$$' prefix.\n"
							+ "Symbols with '$$' prefix are technical symbols which are describe\n"
							+ "at \"New technical-nonterminal-declaration\"\n."
							+ " The declaration ends with ':' and the whole rule ends with ending-symbol ';'."));
			result.add(new CompletionProposal(preSpace + "[] $: ;"
					+ lineSeparator, offset, override, 1 + preSpace.length(),
					null, "Numbered Rule", null,
					"This is the number of the rule. Its an optional information."));
			result.add(new CompletionProposal(
					preSpace + "$$:  ;" + lineSeparator,
					offset,
					override,
					2 + preSpace.length(),
					null,
					"New technical-nonterminal-declaration",
					null,
					"\"Technical symbols\" are nonterminal symbols of the grammar which\n"
							+ "are not relevant for the understanding process but restrict\n"
							+ "word sequences which are allowed by the grammar. They are marked\n"
							+ "in the symbol table so that a parser can use this information\n"
							+ "when generating the parsing output."));
			result.add(new CompletionProposal(
					preSpace + "%IGNORE =  ;" + lineSeparator,
					offset,
					override,
					10 + preSpace.length(),
					null,
					"New ignore-rule",
					null,
					"The \" IGNORE list\" is a list of terminal symbols defined in the\n"
							+ "lexicon. A parser can use this list in order to skip some\n"
							+ "of these words from the word sequence analysed by the parser.\n"
							+ "This may be useful when hesitations like \"er\" should be ignored.\n\n"
							+ "An ignore-rule starts with \"%IGNORE =\"\nfollowed by N terminal-symbols and\n"
							+ "is closed with closing-symbol \";\"\n\nExample:\n %IGNORE= eh ehm ah er;"));
			result.add(new CompletionProposal(
					preSpace + "%IGNORE = eh ehm er ah ahm ;" + lineSeparator,
					offset,
					override,
					29 + preSpace.length(),
					null,
					"Predefined ignore rule.",
					null,
					"Creates a predefined ignore rule using most used ignore words.\n"
							+ "The \" IGNORE list\" is a list of terminal symbols defined in the\n"
							+ "lexicon. A parser can use this list in order to skip some\n"
							+ "of these words from the word sequence analysed by the parser.\n"
							+ "This may be useful when hesitations like \"er\" should be ignored."));
			return (ICompletionProposal[]) result
					.toArray(new ICompletionProposal[result.size()]);
		} catch (Exception e) {
			e.printStackTrace();
			return NO_COMPLETIONS;
		}
	}

	@Override
	public IContextInformation[] computeContextInformation(ITextViewer viewer,
			int offset) {
		return NO_CONTEXTS;
	}

	@Override
	public char[] getCompletionProposalAutoActivationCharacters() {
		return null;
	}

	@Override
	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	@Override
	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}

	@Override
	public String getErrorMessage() {
		return "ERR";
	}
}
