package hterhors.editor;

import hterhors.Activator;
import hterhors.editor.markers.MarkerContainer;
import hterhors.editor.markers.MarkingErrorHandler;
import hterhors.editor.outline.EditorContentOutlinePage;

import java.io.File;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ContentAssistAction;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

/**
 * The main editor. Manages the Plugins Editor.
 * 
 * @author Hendrik
 */
public class ISREditor extends TextEditor {

	/**
	 * @uml.property name="colorManager"
	 * @uml.associationEnd
	 */
	private ColorManager colorManager;
	/**
	 * @uml.property name="input"
	 */
	private IEditorInput input;
	/**
	 * @uml.property name="outlinePage"
	 * @uml.associationEnd
	 */
	private EditorContentOutlinePage outlinePage;

	public ISREditor() {
		super();
		new Activator();
		colorManager = new ColorManager();
		ISRDocumentProvider documentProvider = new ISRDocumentProvider();
		ISRSourceViewerConfiguration svc = new ISRSourceViewerConfiguration(
				colorManager, this);
		setSourceViewerConfiguration(svc);
		setDocumentProvider(documentProvider);
		// if (vT == null) {
		new ValidateThread().start();
		// }
	}

	// static ValidateThread vT = null;
	//
	// private ValidateThread getThread() {
	// if (vT == null) {
	// return vT = new ValidateThread();
	// }
	// return vT;
	//
	// }

	/**
	 * This thread calls validateAndMark() every 500 millisec. to validate the
	 * current Grammar. For efficient the validating is triggered in case of
	 * dirty editor-content.
	 * 
	 * @author Hendrik
	 * 
	 */
	class ValidateThread extends Thread {

		@Override
		public void run() {

			boolean isDirty = true;
			super.run();
			while (true) {
				try {
					isDirty = ISREditor.this.getSite().getPage()
							.getActiveEditor().isDirty()
							&& ISREditor.this.getSite().getPage()
									.getActiveEditor().getTitle() == getInputFile()
									.getName();
				} catch (NullPointerException e) {
				}
				try {
					if (getInputDocument() != null && isDirty)
						validateAndMark();
					sleep(500);
				} catch (InterruptedException e) {
					e.printStackTrace();

				}
			}
		}
	}

	@Override
	public void dispose() {
		colorManager.dispose();
		if (outlinePage != null)
			outlinePage.setInput(null);
		super.dispose();
	}

	@Override
	protected void doSetInput(IEditorInput newInput) throws CoreException {
		super.doSetInput(newInput);
		this.input = newInput;
		if (outlinePage != null)
			outlinePage.setInput(input);
		validateAndMark();
	}

	@Override
	protected void editorSaved() {
		super.editorSaved();
		if (outlinePage != null)
			outlinePage.update();
		this.setInput(input);
	}

	// ValidateThreadListener threadListener = new ValidateThreadListener();

	public synchronized IDocument getInputDocument() {
		try {
			IDocument document = getDocumentProvider().getDocument(input);
			return document;
		} catch (NullPointerException e) {
			return null;
		}
		// if (document != null) {
		// document.addDocumentListener(new IDocumentListener() {
		//
		// @Override
		// public void documentChanged(DocumentEvent event) {
		// System.out.println("changed");
		// // threadListener.run = true;
		// // threadListener.run();
		// }
		//
		// @Override
		// public void documentAboutToBeChanged(DocumentEvent event) {
		// System.out.println("changed2");
		// }
		// });
		// }
	}

	// class ValidateThreadListener extends Thread {
	//
	// boolean run = true;
	//
	// @Override
	// public void run() {
	// super.run();
	// while (run) {
	// System.out.println(run);
	// try {
	// run = false;
	// sleep(2000);
	// } catch (InterruptedException e) {
	// e.printStackTrace();
	// }
	// System.out.println(run);
	// if (getInputDocument() != null && ISREditor.this.isDirty()) {
	// if (!run) {
	// System.out.println("VALIDATE");
	// // validateAndMark();
	// }
	// }
	// }
	// }
	// }

	/**
	 * Triggers the validation of the current grammar.
	 * 
	 */
	public synchronized void validateAndMark() {
		if (getInputDocument() != null
				&& MarkerContainer.isDirty(getInputDocument().get())) {
			MarkingErrorHandler markingErrorHandler = new MarkingErrorHandler(
					getInputFile(), getInputDocument());
			markingErrorHandler.removeExistingMarker();
			markingErrorHandler.setCurrentMarker();
		}
	}

	/**
	 * Returns the current input as an IFile object.
	 * 
	 * @return File with the current grammar.
	 */
	protected synchronized IFile getInputFile() {
		IFileEditorInput ife = (IFileEditorInput) input;
		IFile file = ife.getFile();
		return file;
	}

	/**
	 * Returns the current editor-input
	 * 
	 * @return current editor-input.
	 * @uml.property name="input"
	 */
	public IEditorInput getInput() {
		return input;
	}

	@Override
	protected void createActions() {
		super.createActions();
		ResourceBundle bundle = Activator.getDefault().getResourceBundle();
		setAction("ContentFormatProposal", new TextOperationAction(bundle,
				"ContentFormatProposal.", this, ISourceViewer.FORMAT));

		Action action = new ContentAssistAction(bundle,
				"ContentAssistProposal.", this);
		String id = ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS;
		action.setActionDefinitionId(id);
		setAction("ContentAssistProposal", action);
		markAsStateDependentAction("ContentAssistProposal", true);

	}

	@Override
	public Object getAdapter(Class required) {

		if (IContentOutlinePage.class.equals(required)) {
			if (outlinePage == null) {
				outlinePage = new EditorContentOutlinePage(this);
				if (getEditorInput() != null)
					outlinePage.setInput(getEditorInput());
			}
			return outlinePage;
		}

		return super.getAdapter(required);

	}

}