package hterhors.editor.hyperlink;

import hterhors.editor.ISREditor;
import hterhors.editor.helper.Word;
import hterhors.editor.markers.IsrRuleValidator;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.hyperlink.AbstractHyperlinkDetector;
import org.eclipse.jface.text.hyperlink.IHyperlink;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;


/**
 * @author  Hendrik
 */
public class IsrHyperLinkDetector extends AbstractHyperlinkDetector implements IHyperlinkDetector {

	/**
	 * @uml.property  name="editor"
	 * @uml.associationEnd  
	 */
	private ISREditor editor;

	public IsrHyperLinkDetector() {
	}
	
	public IsrHyperLinkDetector(ISREditor editor) {
		this.editor = editor;
	}

	public IHyperlink[] detectHyperlinks(ITextViewer textViewer,
			IRegion region, boolean canShowMultipleHyperlinks) {
		IDocument document = textViewer.getDocument();
		Word word = new Word(region.getOffset(), document.get());
		if (word.isNull() || !checkValidWord(word.getWord())) {
			return null;
		}
		String text = word.getWord().replace(";", "").replaceAll(":","");
		if (word.getBeginningOffset() < 0 || word.getEndOffset() < 0
				|| word.getBeginningOffset() == word.getEndOffset() + 1) {
			return null;
		}
		IRegion reg = new Region(word.getBeginningOffset() + 1, text
				.length());
		return new IHyperlink[] { new IsrHyperlink(reg, text, editor,
				document) };
	}

	private boolean checkValidWord(String word) {
		return word.matches(IsrRuleValidator.NON_TERMINAL + ";?");
	}

}
