package hterhors.editor.hyperlink;

import java.util.StringTokenizer;

import hterhors.editor.ISREditor;
import hterhors.editor.markers.IsrRuleValidator;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.hyperlink.IHyperlink;

/**
 * Java element hyperlink.
 * @since  3.1
 */
public class IsrHyperlink implements IHyperlink {

	private String word;
	private IRegion region;
	/**
	 * @uml.property  name="editor"
	 * @uml.associationEnd  
	 */
	private ISREditor editor;
	private String document;

	public IsrHyperlink(IRegion region, String text, ISREditor editor,
			IDocument document) {
		this.editor = editor;
		this.region = region;
		this.word = text;
		this.document = document.get();
	}

	public IRegion getHyperlinkRegion() {
		return region;
	}

	public void open() {
		boolean isIndex = false;
		int begin = document.indexOf(word);
		do {
			StringTokenizer stringTokenizer = new StringTokenizer(
					document.substring(begin));
			while (stringTokenizer.hasMoreTokens()) {
				String cT = stringTokenizer.nextToken();
				if (cT.matches(IsrRuleValidator.NON_TERMINAL_DECLARATION)) {
					isIndex = true;
				} else if (cT.matches(IsrRuleValidator.NON_TERMINAL)
						&& stringTokenizer.hasMoreTokens()) {
					if (stringTokenizer.nextToken().matches(IsrRuleValidator.C)) {
						isIndex = true;
					}
				}
				break;
			}
			if (isIndex) {
				break;
			}
		} while ((begin = document.indexOf(word, begin + word.length())) != -1
				|| !isIndex);

		editor.selectAndReveal(begin, word.length() + 1);
		editor.setFocus();
	}

	public String getTypeLabel() {
		return null;
	}

	public String getHyperlinkText() {
		return null;
	}

}