package hterhors.editor.markers;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;

public class IsrRuleValidator {

	public static final String TERMINAL = "([a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*)";
	public static final String IGNORE = "%IGNORE";
	public static final String PRE_START_SYMBOL = "\\$\\$S";
	public static final String NON_TERMINAL_DECLARATION = "(\\${1,2}[a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*:)";
	public static final String NON_TERMINAL = "(\\${1,2}[a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*)";
	public static final String C = ":";
	public static final String E = ";";
	public static final String NUMBER = "\\[-?[0-9]+\\]";
	public static final String WRONG_NUMBER = "\\[.+\\]|\\[.+|.+\\]";
	public static final String O = "(\\|)";
	private static final String S = "(\\$\\$S)";
	private static final String N = "(\\${1,2}[a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*)";
	private static final String J = "((!\\*)|([a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*))";
	private static final String J_T_N = J + "|" + N;
	private static final String JE_TE_NE = "(" + J + ";)|(" + N + ";)";
	static List<ISRValidationObject> errors = new ArrayList<ISRValidationObject>();
	static List<ISRValidationObject> warnings = new ArrayList<ISRValidationObject>();
	protected static final String lineSeparator = System
			.getProperty("line.separator");

	private static IDocument doc;

	public static List<ISRValidationObject> parseErrors(String isrGrammar) {
		errors = new ArrayList<ISRValidationObject>();
		warnings = new ArrayList<ISRValidationObject>();
		doc = new Document(isrGrammar);
		multipleDeclarations(isrGrammar);
		checkValidStartSymbol(isrGrammar);
		// checkWellSpelled(isrGrammar);
		hasAllNonTerminalsDeclared(isrGrammar);
		checkSyntax(isrGrammar);
		return errors;
	}

	public static List<ISRValidationObject> parseWarnings(String isrGrammar) {
		warnUnusedNonterminalDeclaration(isrGrammar);
		warnSameNamedNTAndT(isrGrammar);
		return warnings;
	}

	private static void warnSameNamedNTAndT(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> terminals = new ArrayList<Pair>();
		prepareNonAndTerminals(isrGrammar, nonTerminals, terminals);
		for (Pair pair : terminals) {
			if (nonTerminals.remove(pair)) {
				addWarning(
						"Terminal has same name like an existing noterminal! Rename token ",
						pair.integer, pair.string);
			}
		}
	}

	private static int lastFullLine = 0;

	private static List<StringTokenizer> partition(String isrGrammar) {
		final List<StringTokenizer> partList = new ArrayList<StringTokenizer>();
		for (final String iterable_element : isrGrammar.split(lineSeparator)) {
			partList.add(new StringTokenizer(iterable_element));

		}
		lastFullLine = partList.size();
		for (StringTokenizer stringTokenizer : partList) {
			if (stringTokenizer.countTokens() == 0) {
				lastFullLine--;
			} else {
				break;
			}
		}
		return partList;
	}

	private static void multipleDeclarations(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<StringTokenizer> partList = partition(isrGrammar);
		String preToken = null;
		StringTokenizer stringTokenizer;
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreTokens()) {
				if (preToken == null) {
					preToken = stringTokenizer.nextToken();
				}
				String nextToken;
				if (preToken.matches(NON_TERMINAL)
						&& !preToken.matches(PRE_START_SYMBOL)
						&& stringTokenizer.hasMoreTokens()) {
					nextToken = stringTokenizer.nextToken();
					if (nextToken.equals(C)) {
						if (nonTerminals.contains(new Pair(
								getAttributeName(preToken), 0))) {
							addError(
									"Multiple entry for non-terminal-declaration",
									i, preToken);
						} else {
							nonTerminals.add(new Pair(
									getAttributeName(preToken), 0));
						}
						preToken = null;
					} else {
						preToken = nextToken;
					}
				} else if (preToken.matches(NON_TERMINAL_DECLARATION)
						&& !preToken.matches(PRE_START_SYMBOL + C)) {
					if (nonTerminals.contains(new Pair(
							getAttributeName(preToken), 0))) {
						addError("Multiple entry for non-terminal-declaration",
								i, preToken);
					} else {
						nonTerminals
								.add(new Pair(getAttributeName(preToken), 0));
					}
					preToken = null;
				} else {
					preToken = null;
				}
			}
		}
	}

	private static void checkValidStartSymbol(String isrGrammar) {
		final List<StringTokenizer> partList = partition(isrGrammar);
		String preToken = null;
		int countStartSymbol = 0;
		StringTokenizer stringTokenizer;
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreTokens()) {
				if (preToken == null) {
					preToken = stringTokenizer.nextToken();
				}
				String nextToken;
				if (preToken.matches(PRE_START_SYMBOL + "S:?")) {
					addError("Illigal declaration for nonterminal", i, preToken);
					preToken = null;
				} else if (preToken.matches(PRE_START_SYMBOL)
						&& stringTokenizer.hasMoreTokens()) {
					nextToken = stringTokenizer.nextToken();
					if (nextToken.equals(C)) {
						if (countStartSymbol >= 1) {
							addError("Multiple entry for start-symbol", i,
									preToken);
						} else {
							countStartSymbol++;
						}
						preToken = null;
					} else {
						preToken = nextToken;
					}
				} else if (preToken.matches(PRE_START_SYMBOL + C)) {
					if (countStartSymbol >= 1) {
						addError("Multiple entry for start-symbol", i, preToken);
					} else {
						countStartSymbol++;
					}
					preToken = null;
				} else {
					preToken = null;
				}
			}
		}
		if (countStartSymbol == 0) {
			addError("Missing start-symbol ", 0, "$$S:");
		}
	}

	private static void addError(String message, int i, String token) {
		int offsetStart = exctractOffset(i, token);
		errors.add(new ISRValidationObject(message + " \""
				+ getAttributeName(token) + "\"!", i, offsetStart, offsetStart
				+ token.length(), ISRValidationObject.EValidType.Error));
	}

	private static void hasAllNonTerminalsDeclared(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		for (Pair pair : nonTerminals) {
			if (!decNonTerminals.remove(pair)) {
				addError("Missing Non-Terminal-Declaration for element",
						pair.integer, pair.string);
			}
		}
	}

	static class Pair {
		String string;
		int integer;

		public Pair(String name, int integer) {
			super();
			this.string = name;
			this.integer = integer;
		}

		@Override
		public boolean equals(Object arg0) {
			return string.equals(((Pair) (arg0)).string);
		}

		@Override
		public String toString() {
			return "Pair [string=" + string + ", integer=" + integer + "]";
		}

	}

	private static void warnUnusedNonterminalDeclaration(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		for (Pair pair : decNonTerminals) {
			if (!nonTerminals.remove(pair)) {
				if (!pair.string.equals("SS")) {
					addWarning("Unused Non-Terminal-Declaration for element",
							pair.integer, pair.string);
				}
			}
		}
	}

	private static void addWarning(String message, int i, String token) {
		int offsetStart = exctractOffset(i, token);
		warnings.add(new ISRValidationObject(message + "\"" + token + "\"!", i,
				offsetStart, offsetStart + token.length(),
				ISRValidationObject.EValidType.Warning));
	}

	private static void prepareNonTerminals(String isrGrammar,
			List<Pair> nonTerminals, List<Pair> decNonTerminals) {
		String preToken = null;
		final List<StringTokenizer> partList = partition(isrGrammar);
		StringTokenizer stringTokenizer;
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreElements()) {
				if (preToken == null) {
					preToken = stringTokenizer.nextToken();
				}
				if (preToken.matches(NON_TERMINAL_DECLARATION)
						&& !preToken.matches(PRE_START_SYMBOL + C)) {
					String nt = getAttributeName(preToken);
					if (!decNonTerminals.contains(new Pair(nt, 0))) {
						decNonTerminals.add(new Pair(nt, i));
					}
					preToken = null;
				} else if (preToken.matches(NON_TERMINAL)
						&& !preToken.matches(PRE_START_SYMBOL)) {
					if (stringTokenizer.hasMoreTokens()) {
						String nextToken = stringTokenizer.nextToken();
						String nt = getAttributeName(preToken);
						if (nextToken.equals(C)) {
							if (!decNonTerminals.contains(new Pair(nt, 0))) {
								decNonTerminals.add(new Pair(nt, i));
							}
							preToken = null;
						} else {
							if (!nonTerminals.contains(new Pair(nt, 0))) {
								nonTerminals.add(new Pair(nt, i));
							}
							preToken = nextToken;
						}
					}
				} else if (preToken.matches(NON_TERMINAL + ";")) {
					String nt = getAttributeName(preToken);
					if (!nonTerminals.contains(new Pair(nt, 0))) {
						nonTerminals.add(new Pair(nt, i));
					}
					preToken = null;
				} else {
					preToken = null;
				}
			}
		}
	}

	private static void prepareNonAndTerminals(String isrGrammar,
			List<Pair> nonTerminals, List<Pair> terminals) {
		String preToken = null;
		final List<StringTokenizer> partList = partition(isrGrammar);
		StringTokenizer stringTokenizer;
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreElements()) {
				preToken = stringTokenizer.nextToken();
				if (preToken.matches(TERMINAL + ";?")) {
					String nt = getAttributeName(preToken);
					if (!terminals.contains(new Pair(nt, 0))) {
						terminals.add(new Pair(nt, i));
					}
				} else if (preToken.matches(NON_TERMINAL + ";?")) {
					String nt = getAttributeName(preToken);
					if (!nonTerminals.contains(new Pair(nt, 0))) {
						nonTerminals.add(new Pair(nt, i));
					}
				}
			}
		}
	}

	private static void checkSyntax(String isrGrammar) {
		final List<StringTokenizer> partList = partition(isrGrammar);
		StringTokenizer stringTokenizer;
		String preToken = null;
		boolean afterEnd = true;
		List<Integer> ruleNumbers = new ArrayList<Integer>();
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreElements()) {
				if (preToken == null) {
					preToken = stringTokenizer.nextToken();
				}
				String nextToken;
				if ((preToken.matches(IGNORE + "="))
						|| (preToken.matches(IGNORE)
								&& stringTokenizer.hasMoreElements() && stringTokenizer
								.nextToken().matches("="))) {
					afterEnd = false;
					do {
						if (!stringTokenizer.hasMoreTokens()) {
							i++;
							stringTokenizer = partList.get(i);
						} else {
							nextToken = stringTokenizer.nextToken();
							if (nextToken.matches(TERMINAL)) {
							} else if (nextToken.matches(TERMINAL + ";|" + E)) {
								preToken = nextToken;
								break;
							} else {
								addError("Syntax error on", i, nextToken);
							}
							preToken = nextToken;
						}
					} while (true);
				} else if (preToken.matches(S)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(C)) {
						} else {
							addError("Colon expected after", i, preToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(S + C)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE)) {
						} else {
							addError("Syntax error after \"" + preToken
									+ "\" on token", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (afterEnd && preToken.matches(N)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(C)) {
						} else {
							addError("Colon expected after", i, preToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(N)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE + "|" + E
										+ "|\\|")) {
						} else {
							addError("Syntax error on", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(N + C)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE)) {
							afterEnd = false;
						} else {
							addError("Syntax error after \"" + preToken
									+ "\" on token", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(C)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE)) {
						} else {
							addError("Syntax error after \"" + preToken
									+ "\" on token", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(J_T_N)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE + "|" + E + "|"
										+ O)) {
						} else {
							addError("Syntax error after \"" + preToken
									+ "\" on token", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(O)) {
					afterEnd = false;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken())
								.matches(J_T_N + "|" + JE_TE_NE + "|" + E)) {
						} else {
							addError("Syntax error after \"" + preToken
									+ "\" on token", i, nextToken);
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else if (preToken.matches(JE_TE_NE + "|" + E + "|" + NUMBER)) {
					afterEnd = true;
					if (stringTokenizer.hasMoreTokens()) {
						if ((nextToken = stringTokenizer.nextToken()).matches(S
								+ "|" + N + "|" + S + C + "|" + N + C + "|"
								+ NUMBER + "|" + IGNORE)) {
							if (nextToken.matches(NUMBER)) {
								String x = nextToken.substring(1,
										nextToken.length() - 1);
								Integer number = Integer.valueOf(x);
								if(ruleNumbers.contains(number)){
									addError("Multiple rule-number on token", i,
											nextToken);
								}else{
									ruleNumbers.add(number);
								}
							}
						} else {
							if (nextToken.matches(WRONG_NUMBER)) {
								addError("Not a valid rule-number: ", i,
										nextToken);
							} else {
								addError("New rule expected on token", i,
										nextToken);
							}
						}
						preToken = nextToken;
					} else {
						preToken = null;
					}
				} else {
					// addError("Element not well spelled", i, preToken);
					preToken = null;
				}
			}
			if (preToken != null && i == (partList.size() - 1)
					&& !preToken.matches(JE_TE_NE + "|" + E)) {
				afterEnd = true;
				addError("Valid end-symbol missing. Syntax error on token",
						lastFullLine - 1, preToken);
			}
		}
	}

	private static int exctractOffset(int i, String string) {
		int x = 0;
		try {
			IRegion region = doc.getLineInformation(i);
			String line = doc.get(region.getOffset(), region.getLength());
			x += region.getOffset();
			x += line.indexOf(string);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		return x;
	}

	private static String getAttributeName(String token) {
		if (token.length() != 1) {
			return token.replace("$", "").replace("$$", "").replace(":", "")
					.replace("!*", "").replace(";", "");
		}
		return token;
	}

	public static List<String> getNonterminalsDeclarations(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		List<String> returnvalue = new ArrayList<String>();
		for (Pair pair : decNonTerminals) {
			if (nonTerminals.remove(pair)) {
				returnvalue.add(pair.string);
			}
		}
		return returnvalue;
	}

	public static List<String> getNonterminals(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		List<String> returnvalue = new ArrayList<String>();
		for (Pair pair : nonTerminals) {
			returnvalue.add(pair.string);
		}
		return returnvalue;
	}

	public static List<String> getUnusedNonterminals(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		List<String> returnvalue = new ArrayList<String>();
		for (Pair pair : decNonTerminals) {
			if (!nonTerminals.remove(pair)) {
				returnvalue.add(pair.string);
			}
		}
		return returnvalue;
	}

	public static List<String> getMissingNontermDec(String isrGrammar) {
		final List<Pair> nonTerminals = new ArrayList<Pair>();
		final List<Pair> decNonTerminals = new ArrayList<Pair>();
		prepareNonTerminals(isrGrammar, nonTerminals, decNonTerminals);
		List<String> returnvalue = new ArrayList<String>();
		for (Pair pair : nonTerminals) {
			if (!decNonTerminals.remove(pair)) {
				returnvalue.add(pair.string);
			}
		}
		return returnvalue;
	}

	public static boolean startSymbolExists(String isrGrammar) {
		final List<StringTokenizer> partList = partition(isrGrammar);
		String preToken = null;
		StringTokenizer stringTokenizer;
		for (int i = 0; i < partList.size(); i++) {
			stringTokenizer = partList.get(i);
			while (stringTokenizer.hasMoreTokens()) {
				if (preToken == null) {
					preToken = stringTokenizer.nextToken();
				}
				String nextToken;
				if (preToken.matches(PRE_START_SYMBOL)
						&& stringTokenizer.hasMoreTokens()) {
					nextToken = stringTokenizer.nextToken();
					if (nextToken.equals(C)) {
						return true;
					} else {
						preToken = nextToken;
					}
				} else if (preToken.matches(PRE_START_SYMBOL + C)) {
					return true;
				} else {
					preToken = null;
				}
			}
		}
		return false;
	}
}
