package hterhors.editor.markers;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.IDocument;

/**
 * @author Hendrik
 */
public class MarkingErrorHandler {

	private IFile file;
	private IDocument document;

	public MarkingErrorHandler(IFile file, IDocument document) {
		super();

		this.file = file;
		this.document = document;
	}

	/**
	 * @param document
	 * @uml.property name="document"
	 */
	public void setDocument(IDocument document) {
		this.document = document;
	}

	public synchronized void removeExistingMarker() {
		try {
			file.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_ZERO);
		} catch (CoreException e1) {
			e1.printStackTrace();
		}
	}

	public synchronized void setCurrentMarker() {
		final List<ISRValidationObject> warns = MarkerContainer
				.getWarnings(document.get());
		final List<ISRValidationObject> errs = MarkerContainer
				.getErrors(document.get());
		/*
		 * Avoid ConcurrentModificationException 
		 */
		final ISRValidationObject[] warnsArray;
		warnsArray = warns.toArray(new ISRValidationObject[warns.size()]);
		final ISRValidationObject[] errsArray;
		errsArray = errs.toArray(new ISRValidationObject[errs.size()]);

		for (final ISRValidationObject warn : warnsArray) {
			try {
				final IMarker marker;
				marker = file.createMarker(IMarker.PROBLEM);
				marker.setAttribute(IMarker.LINE_NUMBER,
						warn.getLineNumber() + 1);
				marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
				marker.setAttribute(IMarker.PRIORITY, IMarker.PRIORITY_NORMAL);
				marker.setAttribute(IMarker.MESSAGE, warn.getErrorMessage());
				marker.setAttribute(IMarker.CHAR_START, warn.getColumnStart());
				marker.setAttribute(IMarker.CHAR_END, warn.getColumnEnd());
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		for (final ISRValidationObject error : errsArray) {
			try {
				final IMarker marker;
				marker = file.createMarker(IMarker.PROBLEM);
				marker.setAttribute(IMarker.LINE_NUMBER,
						error.getLineNumber() + 1);
				marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
				marker.setAttribute(IMarker.PRIORITY, IMarker.PRIORITY_HIGH);
				marker.setAttribute(IMarker.MESSAGE, error.getErrorMessage());
				marker.setAttribute(IMarker.CHAR_START, error.getColumnStart());
				marker.setAttribute(IMarker.CHAR_END, error.getColumnEnd());
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}
}