package hterhors.editor.outline;

import hterhors.editor.xml.XMLElement;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;


/**
 * @author  Hendrik
 */
public class EditorContentOutlinePage extends ContentOutlinePage {

	private ITextEditor editor;
	private IEditorInput input;
	/**
	 * @uml.property  name="outlineContentProvider"
	 * @uml.associationEnd  
	 */
	private OutlineContentProvider outlineContentProvider;
	/**
	 * @uml.property  name="outlineLabelProvider"
	 * @uml.associationEnd  
	 */
	private OutlineLabelProvider outlineLabelProvider;

	public EditorContentOutlinePage(ITextEditor editor) {
		super();
		this.editor = editor;
	}

	public void createControl(Composite parent) {

		super.createControl(parent);
		TreeViewer viewer = getTreeViewer();
		outlineContentProvider = new OutlineContentProvider(
				editor.getDocumentProvider());
		viewer.setContentProvider(outlineContentProvider);

		outlineLabelProvider = new OutlineLabelProvider();
		viewer.setLabelProvider(outlineLabelProvider);
		viewer.addSelectionChangedListener(this);

		if (input != null)
			viewer.setInput(input);
	}

	public void setInput(Object input) {
		this.input = (IEditorInput) input;
		update();
	}

	public void selectionChanged(SelectionChangedEvent event) {
		super.selectionChanged(event);
		ISelection selection = event.getSelection();
		if (selection.isEmpty())
			editor.resetHighlightRange();
		else {
			XMLElement element = (XMLElement) ((IStructuredSelection) selection)
					.getFirstElement();

			int start = element.getPosition().getOffset();
			int length = element.getPosition().getLength();
			try {
				editor.setHighlightRange(start, length, true);
			} catch (IllegalArgumentException x) {
				editor.resetHighlightRange();
			}
		}
	}

	public void update() {
		TreeViewer viewer = getTreeViewer();

		if (viewer != null) {
			Control control = viewer.getControl();
			if (control != null && !control.isDisposed()) {
				control.setRedraw(false);
				viewer.setInput(input);
				viewer.expandAll();
				control.setRedraw(true);
			}
		}
	}

}