package hterhors.editor.outline;

import hterhors.editor.xml.XMLElement;
import hterhors.editor.xml.XMLParser;
import hterhors.editor.xmlconverter.IsrFileToXmlFileUsingXOM;

import java.util.List;

import org.eclipse.jface.text.BadPositionCategoryException;
import org.eclipse.jface.text.DefaultPositionUpdater;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IPositionUpdater;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.xml.sax.helpers.LocatorImpl;

/**
 * @author  Hendrik
 */
public class OutlineContentProvider implements ITreeContentProvider {

	/**
	 * @uml.property  name="root"
	 * @uml.associationEnd  
	 */
	private XMLElement root = null;
	private IEditorInput input;
	private IDocumentProvider documentProvider;

	protected final static String TAG_POSITIONS = "__tag_positions";
	protected IPositionUpdater positionUpdater = new DefaultPositionUpdater(
			TAG_POSITIONS);

	public OutlineContentProvider(IDocumentProvider provider) {
		super();
		this.documentProvider = provider;
	}

	public Object[] getChildren(Object parentElement) {
		if (parentElement == input) {
			if (root == null)
				return new Object[0];
			List childrenDTDElements = root.getChildrenDTDElements();
			if (childrenDTDElements != null)
				return childrenDTDElements.toArray();
		} else {
			XMLElement parent = (XMLElement) parentElement;
			List childrenDTDElements = parent.getChildrenDTDElements();
			if (childrenDTDElements != null)
				return childrenDTDElements.toArray();
		}
		return new Object[0];
	}

	public Object getParent(Object element) {
		if (element instanceof XMLElement)
			return ((XMLElement) element).getParent();
		return null;
	}

	public boolean hasChildren(Object element) {
		if (element == input)
			return true;
		else {
			return ((XMLElement) element).getChildrenDTDElements().size() > 0;
		}
	}

	public Object[] getElements(Object inputElement) {
		if (root == null)
			return new Object[0];
		List childrenDTDElements = root.getChildrenDTDElements();
		if (childrenDTDElements != null)
			return childrenDTDElements.toArray();
		return new Object[0];
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

		if (oldInput != null) {
			IsrFileToXmlFileUsingXOM converter = new IsrFileToXmlFileUsingXOM(
					true);
			IDocument xmlDoc = converter.convertFile(documentProvider
					.getDocument(oldInput));
			if (xmlDoc != null) {
				try {
					xmlDoc.removePositionCategory(TAG_POSITIONS);
				} catch (BadPositionCategoryException x) {
				}
				xmlDoc.removePositionUpdater(positionUpdater);
			}
		}
		input = (IEditorInput) newInput;
		if (newInput != null) {
			IsrFileToXmlFileUsingXOM converter = new IsrFileToXmlFileUsingXOM(
					true);
			IDocument xmlDoc = converter.convertFile(documentProvider
					.getDocument(newInput));
			if (xmlDoc != null) {
				xmlDoc.addPositionCategory(TAG_POSITIONS);
				xmlDoc.addPositionUpdater(positionUpdater);
				XMLElement rootElement = parseRootElement(xmlDoc);
				if (rootElement != null) {
					root = rootElement;
				}
			}
		}
	}

	private XMLElement parseRootElement(IDocument document) {
		String text = document.get();
		XMLElement tagPositions = parseRootElements(text, document);
		return tagPositions;
	}

	private XMLElement parseRootElements(String text, IDocument document) {
		try {
			XMLParser xmlParser = new XMLParser();
			OutlineContentHandler contentHandler = new OutlineContentHandler();
			contentHandler.setDocument(document);
			contentHandler.setPositionCategory(TAG_POSITIONS);
			contentHandler.setDocumentLocator(new LocatorImpl());
			xmlParser.setContentHandler(contentHandler);
			xmlParser.doParse(text);
			XMLElement root = contentHandler.getRootElement();
			return root;
		} catch (Exception e) {
			return null;
		}
	}

}
