package hterhors.editor.sentenceparser;

import hterhors.editor.markers.IsrRuleValidator;
import hterhors.editor.sentenceparser.grammar.Grammar;
import hterhors.editor.sentenceparser.grammar.GrammarParser;
import hterhors.editor.sentenceparser.grammar.GrammarParser.ParseException;
import hterhors.editor.sentenceparser.grammar.Tree;
import hterhors.editor.sentenceparser.grammar.TreeFrame;
import hterhors.editor.sentenceparser.grammar.TreeParser;
import hterhors.editor.zest.SentenceGraphBuilder;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.ITextEditor;


/**
 * @author  Hendrik
 */
public class ParseSentenceHandler extends AbstractHandler {

	/**
	 * @uml.property  name="parser"
	 * @uml.associationEnd  
	 */
	private TreeParser parser = null;
	private String result;
	private String defaultString = "Example sentence,Another example";
	/**
	 * @uml.property  name="tree_frame"
	 * @uml.associationEnd  
	 */
	private TreeFrame tree_frame;

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		result = "";
		ITextEditor textEditor = (ITextEditor) HandlerUtil
				.getActiveEditor(event);
		IEditorInput input = HandlerUtil.getActiveEditorInput(event);
		String grm = textEditor.getDocumentProvider().getDocument(input).get();

		Shell shell = HandlerUtil.getActiveShell(event);
		InputDialog dialog = new InputDialog(shell, "Sentence validator!",
				"Enter the sentences to be tested, seperated with commas!",
				defaultString, new IInputValidator() {
					@Override
					public String isValid(String sentences) {
						String x = sentences.replaceAll("\\.*\\?*", "");
						if (x.matches("((" + IsrRuleValidator.TERMINAL
								+ " ?)+,?)+")) {
							return null;
						} else {
							return "Wrong syntax! Sentences must look like e.g.: \"Hello Tobi\" to validate \"Hello Tobi\".\n Multiple sentences must be seperated with commas e.g.: \"Hello Tobi,Go to the kitchen\"!";
						}
					}
				});

		if (dialog.open() == IStatus.OK) {
			String value = defaultString = dialog.getValue();
			MessageBox pmb = new MessageBox(shell, SWT.ICON_QUESTION | SWT.YES
					| SWT.NO);
			pmb.setText("Show Syntax-tree");
			pmb.setMessage("Do you want to display the syntax-trees-graph of the valid given sentences?\n !!ATTENTION: Each sentence will become his own window.!!");
			int val = pmb.open();
			boolean show;
			switch (val) {
			case SWT.YES:
				show = true;
				break;
			default:
				show = false;
				break;
			}
			try {
				if (doParse(parseToCFG(grm), getSentences(value), show)) {
					MessageBox mb = new MessageBox(shell, SWT.ICON_WORKING);
					mb.setText("Result");
					mb.setMessage(result
							+ "\nEvery sentence is valid against the given grammar!");
					mb.open();
				} else {
					MessageBox mb = new MessageBox(shell, SWT.ICON_WARNING);
					mb.setText("Result");
					mb.setMessage(result
							+ "\nOne or more sentences are not valid against the given grammar!");
					mb.open();
				}
			} catch (ParseException e) {
				MessageBox mb = new MessageBox(shell, SWT.ICON_ERROR);
				mb.setText("Result");
				mb.setMessage("An error has accrued, may be the grammar is invalid!");
				mb.open();
			}
		}
		return null;
	}

	private String[] getSentences(String value) {
		String[] split = value.split(",");
		String[] returnValue = new String[split.length];
		for (int i = 0; i < split.length; i++) {
			returnValue[i] = split[i];
		}
		return returnValue;
	}

	public boolean doParse(String grm, String[] sentences, boolean show)
			throws ParseException {
		Grammar g = getGrammar(grm);
		if (g == null)
			throw new GrammarParser.ParseException();
		boolean alright = true;
		SentenceGraphBuilder.clearInput();
		for (int index = 0; index < sentences.length; index++) {
			tree_frame = new TreeFrame();
			StringTokenizer toks = new StringTokenizer(
					removeIgnores(sentences[index].toLowerCase()));
			Grammar.Atom[] text = new Grammar.Atom[toks.countTokens()];
			boolean text_ok = true;
			for (int i = 0; toks.hasMoreTokens(); i++) {
				text[i] = g.getAtom(toks.nextToken());
				if (text[i] == null)
					text_ok = false;
			}
			Tree t = null;
			if (text_ok) {
				parser = new TreeParser(g);
				t = parser.parse(text);
			}
			if (t == null) {
				result += "FAILED!\n -> \"" + sentences[index] + "\"\n\n";
				alright = false;
			} else {
				result += "OK!\n -> \"" + sentences[index] + "\"\n\n";
				if (show) {
					// TreeBuilder x = new TreeBuilder();
					// x.setTree(t, sentences[index]);
					tree_frame.setTree(t, sentences[index]);
					tree_frame.setVisible(true);
				}
			}
		}
		return alright;
	}

	private String removeIgnores(String sentence) {
		for (String string : ignores) {
			sentence = sentence.replaceAll(" " + string + " ", " ");
		}
		return sentence;
	}

	private Grammar getGrammar(String grm) {
		try {
			return GrammarParser.parse(grm);
		} catch (GrammarParser.ParseException e) {
			e.printStackTrace();
			return null;
		}
	}

	private String parseToCFG(String grm) {
		getIgnoreTerminals(grm);
		StringTokenizer stringTokenizer = new StringTokenizer(grm);
		String chomskyGrm = "";
		String start = "";
		boolean isStart = false;
		String token = "";
		boolean nextToken = true;
		while (stringTokenizer.hasMoreTokens()) {
			if (nextToken) {
				token = stringTokenizer.nextToken();
			}
			nextToken = true;
			if (token.matches(IsrRuleValidator.PRE_START_SYMBOL + ":?")) {
				start += "S -> ";
				isStart = true;
			} else if (token.matches(IsrRuleValidator.O)) {
				if (isStart) {
					start += token + " ";
				} else {
					chomskyGrm += token + " ";
				}

			} else if (token.matches(IsrRuleValidator.E)) {
				if (isStart) {
					start += "\n";
				} else {
					chomskyGrm += "\n";
				}

			} else if (token.matches("(!\\*)?" + IsrRuleValidator.TERMINAL)) {
				if (isStart) {
					start += getAttributeName(token).toLowerCase() + " ";
				} else {
					chomskyGrm += getAttributeName(token).toLowerCase() + " ";
				}

			} else if (token.matches("(!\\*)?" + IsrRuleValidator.TERMINAL
					+ ";")) {
				if (isStart) {
					start += getAttributeName(token).toLowerCase() + "\n";
				} else {
					chomskyGrm += getAttributeName(token).toLowerCase() + "\n";
				}
			} else if (token.matches(IsrRuleValidator.NON_TERMINAL_DECLARATION)) {
				isStart = false;
				chomskyGrm += getAttributeName(token).toUpperCase() + " -> ";
			} else if (token.matches(IsrRuleValidator.NON_TERMINAL + ";")) {
				if (isStart) {
					start += getAttributeName(token).toUpperCase() + "\n";
				} else {
					chomskyGrm += getAttributeName(token).toUpperCase() + "\n";
				}

			} else if (stringTokenizer.hasMoreTokens()
					&& token.matches(IsrRuleValidator.NON_TERMINAL)) {
				String oldToken = token;
				nextToken = false;
				if ((token = stringTokenizer.nextToken())
						.matches(IsrRuleValidator.C)) {
					isStart = false;
					chomskyGrm += getAttributeName(oldToken).toUpperCase()
							+ " -> ";
				} else {
					if (isStart) {
						start += getAttributeName(oldToken).toUpperCase() + " ";
					} else {
						chomskyGrm += getAttributeName(oldToken).toUpperCase()
								+ " ";
					}

				}
			} else if (token.matches("%IGNORE")) {
				isStart = false;
				while (stringTokenizer.hasMoreTokens()) {
					if (stringTokenizer.nextToken().matches(
							IsrRuleValidator.E + "|"
									+ IsrRuleValidator.TERMINAL
									+ IsrRuleValidator.E)) {
						break;
					}
				}
			}
		}
		return start + chomskyGrm;
	}

	private List<String> ignores = new ArrayList<String>();

	private void getIgnoreTerminals(String grm) {
		StringTokenizer stringTokenizer = new StringTokenizer(grm);
		String token;
		while (stringTokenizer.hasMoreTokens()) {
			token = stringTokenizer.nextToken();
			if (token.matches("%IGNORE")) {
				String nextToken;
				while (stringTokenizer.hasMoreTokens()) {
					if ((nextToken = stringTokenizer.nextToken())
							.matches(IsrRuleValidator.E + "|"
									+ IsrRuleValidator.TERMINAL
									+ IsrRuleValidator.E)) {
						break;
					} else if (nextToken.matches(IsrRuleValidator.TERMINAL)) {
						ignores.add(nextToken);
					}
				}
			}
		}
	}

	private String getAttributeName(String token) {
		return token.replace("$", "").replace("$$", "").replace(":", "")
				.replace("!*", "").replace(";", "");
	}
}
