/* Copyright (c) 2003, Carl Burch. License information is located in the
 * edu.csbsju.socs.grammar.Main source code and at
 * www.cburch.com/proj/grammar/. */

package hterhors.editor.sentenceparser.grammar;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;

/**
 * @author  Hendrik
 */
public class Grammar {
	private static int last_alloc = 0;

	/**
	 * @author  Hendrik
	 */
	public static class Element {
		/**
		 * @uml.property  name="name"
		 */
		private String name;

		private Element() {
			this.name = "temp" + last_alloc;
			last_alloc++;
		}

		private Element(String name) {
			this.name = name;
		}

		/**
		 * @return
		 * @uml.property  name="name"
		 */
		public String getName() {
			return name;
		}

		public String toString() {
			return name;
		}
	}

	public static class Symbol extends Element {
		public Symbol() {
			super();
		}

		public Symbol(String name) {
			super(name);
		}
	}

	public static class Atom extends Element {
		public Atom() {
			super();
		}

		public Atom(String name) {
			super(name);
		}
	}

	/**
	 * @author  Hendrik
	 */
	public static class Rule {
		/**
		 * @uml.property  name="lhs"
		 * @uml.associationEnd  
		 */
		private Symbol lhs;
		/**
		 * @uml.property  name="rhs"
		 * @uml.associationEnd  multiplicity="(0 -1)"
		 */
		private Element[] rhs;

		public Rule(Symbol lhs, Element[] rhs) {
			this.lhs = lhs;
			this.rhs = rhs;
		}

		public Symbol getLeftSide() {
			return lhs;
		}

		public Element[] getRightSide() {
			return rhs;
		}

		public String toString() {
			StringBuffer ret = new StringBuffer();
			ret.append(lhs + " ->");
			for (int i = 0; i < rhs.length; i++) {
				ret.append(" " + rhs[i]);
			}
			return ret.toString();
		}

	}

	/**
	 * @uml.property  name="root"
	 * @uml.associationEnd  
	 */
	private Symbol root = null;
	/**
	 * @uml.property  name="symbols"
	 */
	private HashSet symbols = new HashSet();
	/**
	 * @uml.property  name="atoms"
	 */
	private HashMap atoms = new HashMap();
	/**
	 * @uml.property  name="rules"
	 */
	private HashSet rules = new HashSet();
	private HashMap rule_map = new HashMap();

	public Grammar() {
	}

	/**
	 * @param root
	 * @uml.property  name="root"
	 */
	public void setRoot(Symbol root) {
		this.root = root;
		symbols.add(root);
	}

	public void addRule(Symbol lhs, Element[] rhs) {
		add(new Rule(lhs, rhs));
	}

	public void add(Rule rule) {
		Collection c = getRules(rule.lhs);
		if (c == null) {
			c = new ArrayList();
			rule_map.put(rule.lhs, c);
		}
		c.add(rule);
		rules.add(rule);

		symbols.add(rule.lhs);
		for (int i = 0; i < rule.rhs.length; i++) {
			Element e = rule.rhs[i];
			if (e instanceof Symbol)
				symbols.add(e);
			else
				atoms.put(e.getName(), e);
		}
	}

	/**
	 * @return
	 * @uml.property  name="root"
	 */
	public Symbol getRoot() {
		return root;
	}

	public Collection getRules(Symbol lhs) {
		return (Collection) rule_map.get(lhs);
	}

	/**
	 * @return
	 * @uml.property  name="rules"
	 */
	public Collection getRules() {
		return rules;
	}

	/**
	 * @return
	 * @uml.property  name="symbols"
	 */
	public Collection getSymbols() {
		return symbols;
	}

	/**
	 * @return
	 * @uml.property  name="atoms"
	 */
	public Collection getAtoms() {
		return atoms.values();
	}

	public Collection getLeftSideSymbols() {
		return rule_map.keySet();
	}

	public Atom getAtom(String name) {
		return (Atom) atoms.get(name);
	}

	public void print(java.io.PrintStream out) {
		Iterator it = rule_map.keySet().iterator();
		while (it.hasNext()) {
			Grammar.Symbol sym = (Grammar.Symbol) it.next();
			boolean first = true;
			Iterator it2 = getRules(sym).iterator();
			while (it2.hasNext()) {
				Rule rule = (Rule) it2.next();
				out.print(first ? (sym + " ->") : "   |");
				first = false;
				Grammar.Element[] rhs = rule.getRightSide();
				for (int i = 0; i < rhs.length; i++) {
					out.print(" " + rhs[i]);
				}
				out.println();
			}
		}
	}
}
