package hterhors.editor.xml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.text.Position;

/**
 * @author  Hendrik
 */
public class XMLElement {

	private List elementChildren = new ArrayList();
	private List attributeChildren = new ArrayList();

	/**
	 * @uml.property  name="name"
	 */
	private String name;
	/**
	 * @uml.property  name="parent"
	 * @uml.associationEnd  
	 */
	private XMLElement parent;
	/**
	 * @uml.property  name="position"
	 */
	private Position position;

	public XMLElement(String name) {
		super();
		this.name = name;
	}

	public List getChildrenDTDElements() {
		return elementChildren;
	}

	public XMLElement addChildElement(XMLElement element) {
		elementChildren.add(element);
		element.setParent(this);
		return this;
	}

	/**
	 * @param element
	 * @uml.property  name="parent"
	 */
	public void setParent(XMLElement element) {
		this.parent = element;
	}

	/**
	 * @return
	 * @uml.property  name="parent"
	 */
	public XMLElement getParent() {
		return parent;
	}

	public XMLElement addChildAttribute(XMLAttribute attribute) {
		attributeChildren.add(attribute);
		return this;
	}

	/**
	 * @return
	 * @uml.property  name="name"
	 */
	public String getName() {
		return name;
	}

	public String getAttributeValue(String localName) {
		for (Iterator iter = attributeChildren.iterator(); iter.hasNext();) {
			XMLAttribute attribute = (XMLAttribute) iter.next();
			if (attribute.getName().equals(localName))
				return attribute.getValue();
		}
		return null;
	}

	public void clear() {
		elementChildren.clear();
		attributeChildren.clear();
	}

	/**
	 * @param position
	 * @uml.property  name="position"
	 */
	public void setPosition(Position position) {
		this.position = position;
	}

	/**
	 * @return
	 * @uml.property  name="position"
	 */
	public Position getPosition() {
		return position;
	}
}