package hterhors.editor.xmlconverter;

import hterhors.editor.ISREditor;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

public class ConvertToXMLAction implements IObjectActionDelegate {

	private Shell shell;

	/**
	 * Constructor for Action1.
	 */
	public ConvertToXMLAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		ISREditor textEditor = (ISREditor) PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		IDocument doc = textEditor.getInputDocument();
		if (doc != null) {
			createOutput(shell, extractResource(textEditor), doc.get());
		} else {
			MessageBox pmb = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
			pmb.setText("An error has accrued!");
			pmb.setMessage("Please try again!");
			pmb.open();
		}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	private QualifiedName path = new QualifiedName("xml", "path");

	private IResource extractResource(IEditorPart editor) {
		// ISelection sel = HandlerUtil.getActiveMenuSelection(event);
		// IStructuredSelection selection = (IStructuredSelection) sel;
		IEditorInput input = editor.getEditorInput();
		if (!(input instanceof IFileEditorInput))
			return null;
		return ((IFileEditorInput) input).getFile();
	}

	private void createOutput(Shell shell, IResource resource, String text) {
		String directory;
		boolean newDirectory = true;
		directory = getPersistentProperty(resource, path);

		if (directory != null && directory.length() > 0) {
			newDirectory = !(MessageDialog.openQuestion(
					shell,
					"Question",
					"Use the previous output directory?"
							+ System.getProperty("line.separator")
							+ System.getProperty("line.separator")
							+ "Attention homonymous file "
							+ System.getProperty("line.separator")
							+ "will be override automatically!"));
		}
		if (newDirectory) {
			DirectoryDialog fileDialog = new DirectoryDialog(shell);
			IWorkspace workspace = ResourcesPlugin.getWorkspace();
			File workspaceDirectory = workspace.getRoot().getLocation()
					.toFile();
			fileDialog.setFilterPath(workspaceDirectory.getAbsolutePath());
			directory = fileDialog.open();

		}
		if (directory != null && directory.length() > 0) {
			setPersistentProperty(resource, path, directory);
			write(directory, text, resource.getName());
		}
	}

	private String getPersistentProperty(IResource res, QualifiedName qn) {
		try {
			return res.getPersistentProperty(qn);
		} catch (CoreException e) {
			return "";
		}
	}

	private void setPersistentProperty(IResource res, QualifiedName qn,
			String value) {
		try {
			res.setPersistentProperty(qn, value);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	private void write(String dir, String text, String name) {
		IsrFileToXmlFileUsingXOM converter = new IsrFileToXmlFileUsingXOM();
		try {
			String n;
			try {
				n = name.split("\\.")[0];
			} catch (ArrayIndexOutOfBoundsException e) {
				n = name;
			}
			String xmlFile = dir + "\\" + n + ".xml";
			FileWriter output = new FileWriter(xmlFile);
			BufferedWriter writer = new BufferedWriter(output);
			writer.write(converter.convertText(text));
			writer.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}