package hterhors.editor.xmlconverter;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import nu.xom.*;

import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;

public class IsrFileToXmlFileUsingXOM {

	private static final String TERMINAL = "([a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*)";
	private static final String IGNORE = "%IGNORE";
	private static final String C = ":";
	private static final String E = ";";
	private static final String O = "(\\|)";
	private static final String S = "(\\$\\$S)";
	private static final String N = "(\\${1,2}[a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*)";
	private static final String J_T = "((!\\*)?([a-zA-Z_\\\"][a-zA-Z_\\-\\\"0-9]*))";
	private static final String J_T_N = J_T + "|" + N;
	private static final String JE_TE_NE = "(" + J_T + ";)|(" + N + ";)";

	private List<String> ignores = new ArrayList<String>();

	private String xmlString = "<!DOCTYPE world SYSTEM \"IsrGrammarXmlDefinition.dtd\">\n\n<ISR_GRAMMAR>\n";
	private boolean addIgnore = true;
	private Element ignore_one_of;
	private Element root = new Element("grammar");
	private boolean forContentOutline = false;

	public IsrFileToXmlFileUsingXOM(boolean forContentOutline) {
		this.forContentOutline = forContentOutline;
	}

	public IsrFileToXmlFileUsingXOM() {
	}

	public String convertText(String isrGrammar) {
		if (isrGrammar != null) {
			write(new StringTokenizer(isrGrammar));
		}
		// root.setBaseURI("http://www.w3.org/2001/06/grammar");
		// ("version", "1.0");
		root.addAttribute(new Attribute("version", "1.0"));
		root.addAttribute(new Attribute("root", "S"));
		root.addAttribute(new Attribute("mode", "voice"));
//		root.addAttribute(new Attribute("xmlns",
//				"http://www.w3.org/2001/06/grammar/"));
		//
//		root.addNamespaceDeclaration("xmlns",
//				"http://www.w3.org/2001/06/grammar/");
		// root.addNamespaceDeclaration("xmlns:xsi",
		// "http://www.w3.org/2001/XMLSchema-instance");
		// root.addNamespaceDeclaration("xsi:schemaLocation",
		// "http://www.w3.org/2001/06/grammar\n"
		// + "http://www.w3.org/TR/speech-grammar/grammar.xsd");
		// root.addNamespaceDeclaration("xml:lange", "en-US");
		// root.addNamespaceDeclaration("mode", "voice");
		// root.addNamespaceDeclaration("root", "Start");
		nu.xom.Document doc = new nu.xom.Document(root);
		String result = doc.toXML();
		return result;
	}

	public IDocument convertFile(IDocument document) {
		return new Document(convertText(document.get()));
	}

	void write(StringTokenizer stringTokenizer) {
		try {
			String token;
			while (stringTokenizer.hasMoreElements()) {
				token = stringTokenizer.nextToken();
				if ((token.matches(IGNORE + "="))
						|| (token.matches(IGNORE)
								&& stringTokenizer.hasMoreElements() && stringTokenizer
								.nextToken().matches("="))) {
					if (addIgnore) {
						addIgnore = false;
						Element ignore = new Element("rule");
						ignore.addAttribute(new Attribute("id", "Ignore"));
						ignore.appendChild(ignore_one_of = new Element("one-of"));
						root.appendChild(ignore);
					}
					do {
						String element = stringTokenizer.nextToken();
						if (!element.matches(TERMINAL + ";|" + E)) {
							if (element.matches(TERMINAL)) {
								if (!ignores.contains(element)) {
									ignores.add(getAttributeName(element));
								}
							}
						}
						if (element.matches(TERMINAL + ";|" + E)) {
							if (!getAttributeName(element).isEmpty()
									&& !ignores.contains(element)) {
								ignores.add(getAttributeName(element));
							}
							break;
						}
					} while (stringTokenizer.hasMoreTokens());
				} else if (token.matches(S + C)
						|| (token.matches(S)
								&& stringTokenizer.hasMoreElements() && stringTokenizer
								.nextToken().matches(C))) {
					Element start = new Element("rule");
					start.addAttribute(new Attribute("id", "Start"));
					root.appendChild(start);
					Element oneOf = new Element("one-of");
					Element item;
					oneOf.appendChild(item = new Element("item"));
					start.appendChild(oneOf);
					do {
						String element = stringTokenizer.nextToken();
						if (!element.matches(E)) {
							if (element.matches(N + ";?")) {
								Element nt = new Element("ruleref");
								nt.addAttribute(new Attribute("uri", "#"
										+ getAttributeName(element)));
								item.appendChild(nt);
							} else if (element.matches(TERMINAL + ";?")) {
								item.appendChild(getAttributeName(element)
										+ " ");
							}
						}
						if (element.matches(O)) {
							oneOf.appendChild(item = new Element("item"));
						}
						if (element.matches(JE_TE_NE + "|" + E)) {
							break;
						}
					} while (stringTokenizer.hasMoreTokens());
				} else if (token.matches(N + C)
						|| (token.matches(N)
								&& stringTokenizer.hasMoreElements() && stringTokenizer
								.nextToken().matches(C))) {
					Element start = new Element("rule");
					start.addAttribute(new Attribute("id",
							getAttributeName(token)));
					root.appendChild(start);
					Element oneOf = new Element("one-of");
					Element item;
					oneOf.appendChild(item = new Element("item"));
					start.appendChild(oneOf);
					do {
						String element = stringTokenizer.nextToken();
						if (element.matches(O)) {
							oneOf.appendChild(item = new Element("item"));
						} else if (!element.matches(E)) {
							if (element.matches(N + ";?")) {
								Element nt = new Element("ruleref");
								if (forContentOutline) {
									nt.addAttribute(new Attribute("id",
											getAttributeName(element)));
								} else {
									nt.addAttribute(new Attribute("uri", "#"
											+ getAttributeName(element)));
								}
								item.appendChild(nt);
							} else if (element.matches(TERMINAL + ";?")) {
								if (forContentOutline) {
									Element t = new Element("token");
									t.addAttribute(new Attribute("id",
											getAttributeName(element)));
									item.appendChild(t);
								} else {
									item.appendChild(getAttributeName(element)
											+ " ");
								}
							}
						}
						if (element.matches(JE_TE_NE + "|" + E)) {
							break;
						}
					} while (stringTokenizer.hasMoreTokens());
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		Element item;
		for (String ignore : ignores) {
			ignore_one_of.appendChild(item = new Element("item"));
			if (forContentOutline) {
				Element t = new Element("token");
				t.addAttribute(new Attribute("id", getAttributeName(ignore)));
				item.appendChild(t);
			} else {
				item.appendChild(getAttributeName(ignore));
			}
		}
	}

	private String getAttributeName(String token) {
		return token.replace("$", "").replace("$$", "").replace(":", "")
				.replace("!*", "").replace(";", "");
	}

}
