package hterhors.editor.zest;

import hterhors.editor.IISRColorConstants;
import hterhors.editor.markers.IsrRuleValidator;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.zest.core.widgets.Graph;
import org.eclipse.zest.core.widgets.GraphConnection;
import org.eclipse.zest.core.widgets.GraphNode;
import org.eclipse.zest.core.widgets.ZestStyles;
import org.eclipse.zest.layouts.LayoutAlgorithm;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.CompositeLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.GridLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.HorizontalShift;
import org.eclipse.zest.layouts.algorithms.HorizontalTreeLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.RadialLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.SpringLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.TreeLayoutAlgorithm;
import org.eclipse.zest.layouts.algorithms.VerticalLayoutAlgorithm;

/**
 * @author  Hendrik
 */
public class IsrGraphBuilder {

	private static String isrGrammar;

	public static void getGraph(Composite parent) {
		if (isrGrammar != null) {
			Device device = parent.getDisplay().getSystemColor(SWT.COLOR_GREEN)
					.getDevice();
			List<String> graphNodes = new ArrayList<String>();
			final Graph graph = new Graph(parent, SWT.NONE);
			StringTokenizer ruleToken = new StringTokenizer(isrGrammar, ";",
					false);
			boolean isIgnoreRule;
			boolean isEmptyRule;
			while (ruleToken.hasMoreTokens()) {
				isIgnoreRule = false;
				isEmptyRule = false;
				GraphNode currentStartNode = null;
				GraphNode lastNode = null;
				String ruleText = ruleToken.nextToken();
				if (!ruleText.trim().isEmpty()) {
					lastNode = new GraphNode(graph, SWT.NONE, ";");
					lastNode.setBackgroundColor(new Color(device, 230, 230, 230));
				} else {
					isEmptyRule = true;
				}
				StringTokenizer orRulePartToken = new StringTokenizer(ruleText,
						"|", false);
				while (orRulePartToken.hasMoreTokens()) {
					StringTokenizer wordToken = new StringTokenizer(
							orRulePartToken.nextToken());
					int tokenCount = wordToken.countTokens();
					graphNodes.clear();
					String tmp;
					while (wordToken.hasMoreTokens()) {
						if ((tmp = wordToken.nextToken()).matches(":|=")) {
						} else {
							if (tmp.charAt(0) == '%') {
								tmp = tmp.replace("=", "");
								graphNodes.add(tmp + "=");
								isIgnoreRule = true;
							} else if (tmp.charAt(0) == '$') {
								if (tmp.contains(":")) {
									tmp = tmp.replace(":", "");
									graphNodes.add(tmp + ":");
								} else {
									graphNodes.add(tmp);
								}
							} else {
								graphNodes.add(tmp);
							}
						}
					}
					GraphNode gn1 = null;
					GraphNode gn2 = null;
					for (int i = 0; i < graphNodes.size(); i++) {
						if (isIgnoreRule) {
							if (gn1 == null) {
								gn1 = new GraphNode(graph, SWT.NONE,
										graphNodes.get(i));
								currentStartNode = gn1;
								currentStartNode
										.setBackgroundColor(new Color(device,
												IISRColorConstants.IGNORE_GRAPH));
							} else {
								try {
									gn2 = new GraphNode(graph, SWT.NONE,
											graphNodes.get(i + 1));
									gn2.setBackgroundColor(new Color(device,
											255, 200, 200));
									GraphConnection gc = new GraphConnection(
											graph,
											ZestStyles.CONNECTIONS_DIRECTED,
											currentStartNode, gn2);
									gc.setLineColor(new Color(device,
											IISRColorConstants.OR));
									new GraphConnection(graph,
											ZestStyles.CONNECTIONS_DIRECTED,
											gn2, lastNode);
								} catch (IndexOutOfBoundsException e) {
								}
							}
						} else {
							if (gn2 == null) {
								gn1 = new GraphNode(graph, SWT.NONE,
										graphNodes.get(i));
								if (graphNodes.get(i).matches(
										IsrRuleValidator.TERMINAL + ";?")) {
									gn1.setBackgroundColor(new Color(device,
											255, 255, 255));
								} else if (graphNodes.get(i).matches(
										"(!\\*)" + IsrRuleValidator.TERMINAL
												+ ";?")) {
									gn1.setBackgroundColor(new Color(device,
											255, 255, 255));
								} else {
									if (graphNodes.get(i).matches(
											IsrRuleValidator.PRE_START_SYMBOL
													+ ":?")) {
										gn1.setBackgroundColor(new Color(
												device,
												IISRColorConstants.START_GRAPH));
									} else if (graphNodes.get(i).matches(
											IsrRuleValidator.IGNORE + "=?")) {
										gn1.setBackgroundColor(new Color(
												device,
												IISRColorConstants.IGNORE_GRAPH));
									} else {
										gn1.setBackgroundColor(new Color(
												device,
												IISRColorConstants.DEC_NON_TERMINAL_GRAPH));
									}
								}
								if (currentStartNode == null) {
									currentStartNode = gn1;
								} else {
									GraphConnection gc = new GraphConnection(
											graph,
											ZestStyles.CONNECTIONS_DIRECTED,
											currentStartNode, gn1);
									gc.setLineColor(new Color(device,
											IISRColorConstants.OR_GRAPH));
								}
							} else {
								gn1 = gn2;
								setColor(device, graphNodes.get(i), gn1);
							}
							try {
								gn2 = new GraphNode(graph, SWT.NONE,
										graphNodes.get(i + 1));
								setColor(device, graphNodes.get(i), gn2);
							} catch (IndexOutOfBoundsException e) {
								gn2 = lastNode;
							}
							if (gn2 != null) {
								GraphConnection gc = new GraphConnection(graph,
										ZestStyles.CONNECTIONS_DIRECTED, gn1,
										gn2);
								if (gn1 == currentStartNode
										&& orRulePartToken.countTokens() == 1) {
									gc.setLineColor(new Color(device,
											IISRColorConstants.OR_GRAPH));
								}
							}
						}
					}

					if (tokenCount == 0 && !isEmptyRule) {
						GraphConnection gc = new GraphConnection(graph,
								ZestStyles.CONNECTIONS_DIRECTED,
								currentStartNode, lastNode);
						gc.setLineColor(new Color(device,
								IISRColorConstants.OR_GRAPH));
					}
				}
			}

			switch (layout) {
			case TreeLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new TreeLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			case HorizontalTreeLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new HorizontalTreeLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			case RadialLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new RadialLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			case GridLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new GridLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			case VerticalLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new VerticalLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			case SpringLayoutAlgorithm:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new SpringLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
				break;
			default:
				graph.setLayoutAlgorithm(
						new CompositeLayoutAlgorithm(
								LayoutStyles.NO_LAYOUT_NODE_RESIZING,
								new LayoutAlgorithm[] {
										new HorizontalTreeLayoutAlgorithm(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING),
										new HorizontalShift(
												LayoutStyles.NO_LAYOUT_NODE_RESIZING) }),
						true);
			}
		}
	}

	private static void setColor(Device device, String nodeText, GraphNode node) {
		if (nodeText.matches(IsrRuleValidator.NON_TERMINAL + ";?")) {
			node.setBackgroundColor(new Color(device,
					IISRColorConstants.NON_TERMINAL_GRAPH));
		} else if (nodeText.matches(IsrRuleValidator.IGNORE + "=?")) {
			node.setBackgroundColor(new Color(device,
					IISRColorConstants.IGNORE_GRAPH));
		} else if (nodeText
				.matches("(!\\*)" + IsrRuleValidator.TERMINAL + ";?")) {
			node.setBackgroundColor(new Color(device,
					IISRColorConstants.IGNORE_GRAPH));
		} else {
			node.setBackgroundColor(new Color(device, 255, 255, 255));
		}
	}

	/**
	 * @uml.property  name="layout"
	 * @uml.associationEnd  
	 */
	private static ELayouts layout = ELayouts.HorizontalTreeLayoutAlgorithm;

	public static void setInput(String isrGrammar, ELayouts l) {
		IsrGraphBuilder.isrGrammar = remNumbers(isrGrammar);
		if (l != null)
			layout = l;
	}

	private static String remNumbers(String isrGrammar2) {
		return isrGrammar2.replaceAll(IsrRuleValidator.NUMBER, "");
	}
}
