package hterhors.editor.zest;

import hterhors.editor.markers.IsrRuleValidator;

import java.util.StringTokenizer;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * @author  Hendrik
 */
public class SetGraphSettingsHandler extends AbstractHandler {
	/**
	 * @uml.property  name="layout"
	 * @uml.associationEnd  
	 */
	public static ELayouts layout;

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		generateInput(event);
		IWorkbenchPage page = HandlerUtil.getActiveWorkbenchWindow(event)
				.getActivePage();
		IViewPart findView = (IViewPart) page
				.findView("hterhors.editor.zest.IsrGraphView");
		page.hideView(findView);
		try {
			page.showView("hterhors.editor.zest.IsrGraphView");
		} catch (PartInitException e) {
		}
		return null;
	}

	/**
	 * @param l
	 * @uml.property  name="layout"
	 */
	public void setLayout(ELayouts l) {
		layout = l;
	}

	private void generateInput(ExecutionEvent event) {
		ITextEditor textEditor = (ITextEditor) HandlerUtil
				.getActiveEditor(event);
		IEditorInput input = HandlerUtil.getActiveEditorInput(event);
		ITextSelection selection = (ITextSelection) textEditor.getEditorSite()
				.getSelectionProvider().getSelection();
		if (selection.getText().trim().isEmpty()) {
			IsrGraphBuilder.setInput(textEditor.getDocumentProvider()
					.getDocument(input).get(), layout);
		} else {
			if (testSelectionIsARule(selection.getText().trim())) {
				IsrGraphBuilder.setInput(selection.getText(), layout);
			} else {
				Shell shell = (Shell) HandlerUtil.getActiveShell(event);
				MessageBox pmb = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
				pmb.setText("Unvalid selection");
				pmb.setMessage("The selection you did, was not a valid construct and cannot be displayed!");
				pmb.open();
			}
		}
	}

	private boolean testSelectionIsARule(String selection) {
		StringTokenizer tokenizer = new StringTokenizer(selection);
		String preToken = tokenizer.nextToken();
		if (selection.trim().endsWith(";")) {
			if ((preToken.matches(IsrRuleValidator.IGNORE + "="))
					|| (preToken.matches(IsrRuleValidator.IGNORE)
							&& tokenizer.hasMoreElements() && tokenizer
							.nextToken().matches("="))) {
				return true;
			} else if (((preToken.matches(IsrRuleValidator.NUMBER) && tokenizer
					.hasMoreElements()) && (((preToken = tokenizer.nextToken())
					.matches(IsrRuleValidator.NON_TERMINAL + ":")) || (preToken
					.matches(IsrRuleValidator.NON_TERMINAL)
					&& tokenizer.hasMoreElements() && tokenizer.nextToken()
					.matches(":"))))
					|| ((preToken.matches(IsrRuleValidator.NON_TERMINAL + ":")) || (preToken
							.matches(IsrRuleValidator.NON_TERMINAL)
							&& tokenizer.hasMoreElements() && tokenizer
							.nextToken().matches(":")))) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

}
